function [meta] = get_deep_CNN(meta, filenames, total_entries)

%{
@article{arXiv:1412.4564,
     author    = {A. Vedaldi and K. Lenc},
     title     = {MatConvNet -- Convolutional Neural Networks for MATLAB},
     journal   = {CoRR},
     volume    = {abs/1412.4564},
     year      = {2014},
%}

%Read each image from the folder and get its very deep CNN descriptor and
%classification

%load('state.mat');

% setup MatConvNet
run matconvnet-1.0-beta12/matlab/vl_setupnn

% load the pre-trained CNN
%net = load('matconvnet-1.0-beta12/imagenet-vgg-verydeep-16.mat');
net = load('matconvnet-1.0-beta12/imagenet-vgg-verydeep-16.mat');

for e = 1 : total_entries
    meta(e).goodImg = false;
    try
        tic;
        im = imread(strtrim(filenames(e,:)));  %Matlab imread can also fetch from a URL
        im_ = single(im); %note: 255 range
        im_ = imresize(im_, net.normalization.imageSize(1:2));
        im_ = im_ - net.normalization.averageImage;
        res = vl_simplenn(net, im_);
        t1 = toc;

        tic;
        meta(e).X_deep = res(36).x;        % Store the intermediate layer result res(36).x in meta
        meta(e).X_deep_end = res(end).x;   % Store the classification according to vgg in meta
        t2 = toc;
        fprintf(1, 'Image %d/%d, time %1.0f+%1.0f=%1.0f sec, ETA: %2.0f sec\n', e, total_entries, t1, t2, t1+t2, (total_entries-e)*(t1-t2));
        meta(e).goodImg = true;
    catch
        disp('Caught error');
    end
end

% Show the classification result
scores = squeeze(gather(res(end).x));
[bestScore, bestCat] = max(scores);
figure(1); clf; imagesc(im);
title(sprintf('%s (%d), score %.3f',net.classes.description{bestCat}, bestCat, bestScore));

% Put the best scoring category and its score in meta
for e = 1:total_entries
    [bestScore, bestCat] = max(squeeze(meta(e).X_deep_end));
    meta(e).bestScore = bestScore;
    meta(e).bestCatID = bestCat;
    meta(e).bestCat = net.classes.description{bestCat};
end
