function get_deep_folder(root, net)

%{
@article{arXiv:1412.4564,
     author    = {A. Vedaldi and K. Lenc},
     title     = {MatConvNet -- Convolutional Neural Networks for MATLAB},
     journal   = {CoRR},
     volume    = {abs/1412.4564},
     year      = {2014},
%}

split = strsplit(root, '/');
folder = char(split(end));
outFileName = ['/dist_vision_scratch/InstagramFood/deep/',folder,'_deep'];

deep.folder = root;
d = dir([deep.folder, '/', '*.jpg']);
N = numel(d);
deep.X     = zeros(N,4096);
deep.X_end = zeros(N,1000);
deep.good  = false(N,1);
deep.name  = cell(N,1);
deep.timeStart = datestr(now);
[~,deep.host] = system('hostname');

count_save = 5000; %save the features for backup at each 5000th
count = 0;

for e = 1 : N
    try
        tic;
        % load and preprocess an image
        deep.name{e} = d(e).name;
        %im = imread(meta(e).url);
        im = imread([root, '/', deep.name{e}]);
        im_ = single(im); % note: 255 range
        im_ = imresize(im_, net.normalization.imageSize(1:2));
        im_ = im_ - net.normalization.averageImage;
        res = vl_simplenn(net, im_);
        t1 = toc;
        
        tic;
        deep.X(e,:) = res(36).x; % the features
        deep.X_end(e,:) = res(end).x; % the classification according to vgg
        [bestScore, bestCat] = max(squeeze(deep.X_end(e,:)));
        deep.bestScore(e) = bestScore;
        deep.bestCatID(e) = bestCat;
        deep.bestCat{e} = net.classes.description{bestCat};
        
        t2 = toc;
        fprintf(1,'Image %d/%d, time %1.0f+%1.0f=%1.0f sec, ETA: %2.0f sec\n', e, N, t1, t2, t1+t2, (N-e)*(t1+t2));
        deep.good(e) = true;
    catch
      fprintf(1,'Caught error img %d\n', e);           %Will throw an error for images that have been deleted.
%									  keyboard;
    end
    count = count + 1;
    if (mod(count, count_save) == 0 || count == N)
        save([outFileName,'_',num2str(count),'.mat'],'deep','-v7.3')
    end
end

deep.timeEnd = datestr(now);
save(outFileName,'deep','-v7.3')


