function local_tag_correlation

%% Get the correlation between visual tags
load('tagDict.mat');
load('visualBitsMatAll.mat');

tags = [224, 34, 277, 97, 32, 451, 41, 62, 228, 646, 43, 151, 306, ...
    72, 84, 308, 226, 719, 78, 125, 651, 387];
hashtags = sort(tags);

C1 = cov(visualBitsMatAll);
figure;
imagesc(C1); colorbar;
[sc,sci]=sort(C1(:),'descend');
[i_idx,j_idx]=ind2sub([numel(hashtags),numel(hashtags)],sci);

for e = 1:(numel(hashtags)^2)
    if(~(i_idx(e)==j_idx(e)))
        cor(e,1) = {tagDict(hashtags(i_idx(e)))};
        cor(e,2) = {tagDict(hashtags(j_idx(e)))};
        cor(e,3) = {sc(e)};
    end
end
notEmpty = find(cellfun(@(x)~isempty(x),cor(:,1)));
cor_csv = cor(notEmpty,:); % For correlation chord diagram

%% Get the correlations between the visual tags across continents

load('visualBitsMatAll_spatial.mat');

fields = {'as', 'af', 'au', 'eu', 'na', 'sa'};
continents = {'Asia', 'Africa','Australia','Central America and the Caribbean', 'Europe','North America','South America'};

for g = 1:numel(fields)
    C2.(char(fields(g))) = cov(visualBitsMat_spatial.(char(fields(g))));
    figure;
    imagesc(C2.(char(fields(g)))); colorbar;
    C2.(char(fields(g))) = C2.(char(fields(g))).*(1-eye(22)); imagesc(C2.(char(fields(g)))); colorbar;
    [sc.(char(fields(g))),sci.(char(fields(g)))]=sort(C2.(char(fields(g)))(:),'descend');
    [i_idx.(char(fields(g))),j_idx.(char(fields(g)))]=ind2sub([numel(hashtags),numel(hashtags)],sci.(char(fields(g))));
    
    for e = 1:(numel(hashtags)^2)
        if(~(i_idx.(char(fields(g)))(e)==j_idx.(char(fields(g)))(e)))
            cor2.(char(fields(g)))(e,1) = {tagDict(hashtags(i_idx.(char(fields(g)))(e)))};
            cor2.(char(fields(g)))(e,2) = {tagDict(hashtags(j_idx.(char(fields(g)))(e)))};
            cor2.(char(fields(g)))(e,3) = {sc.(char(fields(g)))(e)};
        end
    end
    notEmpty.(char(fields(g))) = find(cellfun(@(x)~isempty(x),cor2.(char(fields(g)))(:,1)));
    cor2_csv.(char(fields(g))) = cor2.(char(fields(g)))(notEmpty.(char(fields(g))),:); % For chord diagram
end

%% Get the frequency of these visual tags across the different continents

% Overall sum
sumBits = sum(visualBitsMatAll, 1);
[sorted_all, idx_all] = sort(sumBits, 'descend');

% Prints the top 20 visual tags
for e = 1:20
    tagDict(hashtags(idx_all(e)))
end

% By continent
for f = 1:numel(fields)
    sumBitsSpatial.(char(fields(f))) = sum(visualBitsMat_spatial.(char(fields(f))), 1);
    [sorted_spatial.(char(fields(f))), idx_spatial.(char(fields(f)))] = sort(sumBitsSpatial.(char(fields(f))), 'descend');
end
   
% pop_tags contains the visual tags in order of popularity by continent
for f = 1:numel(fields)
    for e = 1:numel(hashtags)
        pop_tags.(char(fields(f)))(e,1) = tagDict(hashtags(idx_spatial.(char(fields(f)))(e)));
    end
end
save('/Users/Jackie/Dropbox/QMUL_Final_Project/local_copies/pop_tags.mat', 'pop_tags');

