%% Main project file

%% Open and process the Metadata file into a structure array, then close file.

% Process all the metadata files from all folders
% Each folder's structure array is saved in
% /import/geb-datasets/InstagramFood/images/FOLDERNAME
parseHashDataset();

%% Get tagBits for the whole dataset
% The tag list for each meta is saved in /scratch/InstagramFood/tags_lists/FOLDERNAME_tags_list.mat
getTagsListDataset(); 

% The combined tagslist /scratch/InstagramFood/tags_lists/tags_list.mat
% Also created tagDict of the top 10,000 tags - saved in /scratch/InstagramFood/tagDict.mat
combineTagLists(); 

% Save /scratch/InstagramFood/tags_lists/tags_list.mat locally to run the code below
% Run locally - plot frequency of top 1000 and 10000 tags
load(['/dist_vision_scratch/InstagramFood/tags_lists/', 'tags_list.mat']);
tbl = tabulate(tags_list);
fprintf(1, 'Number of unique tags: %d\n', length(tbl));
tblsort = sortrows(tbl,[-2, 1]);
tagFreqs = tblsort(:,2);
plot(1:1000,cell2mat(tagFreqs(1:1000)));
plot(1:10000,cell2mat(tagFreqs(1:10000)));

% Gets tagBits (binary indicator matrix for tags) for each meta and saves
% the new meta in folder /scratch/InstagramFood/withTagBits/FOLDER_withTagBits.mat
getTagBitsDataset();

%% Reverse geocode the dataset

% Creates csvs of each meta's GPS coordinates
% These are then reverse geocoded in R
getGPSTextFile();

% Saves the country and continent found in R to each meta
% Reverse geocodes the NA's using Google API
getAddress();
addAddresses();

% Gets the top 100 countries, and the breakdown of posts by continent
spatialBreakdown();

% Save /scratch/InstagramFood/reverse_geocoding/all_countries.mat locally
% to run the code below
% Run locally - gets histogram and list of country frequencies
load('localResults/ReverseGeocoding/all_countries.mat');
[idx,uniqueCountries] = grp2idx(sort(all_countries));
hist(idx,unique(idx));
set(gca,'xTickLabel',uniqueCountries,'XTick',1:numel(uniqueCountries),'TickLength', [0,0])
[freq,edges] = histcounts(idx,numel(uniqueCountries));
[countryFreq,countryIdx] = sort(freq, 'descend');
    % Prints the countries : their frequency
    for t = 1:numel(uniqueCountries)
        fprintf(1, '%d : %s : %d\n', t, char(uniqueCountries(countryIdx(t))), countryFreq(t));
    end

%% Get each image, get very deep CNN descriptor and classification for each image, and store them in meta.

% Gets the features for each meta and saves them in /scratch/InstagramFood/FOLDER_deep.mat
get_deep_dataset();

% Add the deep features to each meta
% Each meta from /scratch/InstagramFood/withTagBits/FOLDER_withTagBits.mat
% is joined with its deep features and then the new meta is saved in
% /scratch/InstagramFood/meta_with_tagBits_GPS_Deep/FOLDERNAME_withTagBits_deep_meta.mat
combineFolder(); %<- Currently running this one!!!

%% Save certain columns of all metas separately
get_meta_cols();

%% Analyze VGG classification

% Print histogram about the most frequent classes as IDed by vggnet
% Save /scratch/InstagramFood/vgg/classes.mat and
% /scratch/InstagramFood/meta_cols/vgg_catsAll.mat locally to run the
% function below
vgg_classes_histogram();

% Creates toView.mat a structure array with fields being the food classes (topFoods)
% that has the top fifty scoring images (according to VGG) and their urls and
% filenames
get_top_scoring_vgg();

% Visualize the top ten most confident images per food class
visualize_top_scoring_vgg

% Creates toViewDist.mat a structure array with fields being the possible
% distractor classes (topDist) that has the top fifty scoring images
% (according to VGG) and their urls and filenames
get_top_scoring_vgg_distractor();

% Visualize the top twenty most confident images per possible
% distractor class
visualize_top_scoring_vgg_distractor

%% Get X and Y ready to be used by the classifiers
get_all_X_Y();

%%
% Similar function to get_meta_cols - creates binary indicator vector
% for notDistractClass, food class, plate or bowl class and vectors
% of the indices of their occurrences in meta
set_distract_class();

%% Train a classifier: for a tag - no filtering

% Balance training and testing sets and run classifiers
load('/dist_vision_scratch/InstagramFood/x_and_y/Xtr.mat');
load('/dist_vision_scratch/InstagramFood/x_and_y/Xte.mat');
load('/dist_vision_scratch/InstagramFood/x_and_y/Y.mat');
resultsFolder = 'results_raw_classifier';
for hashtag = 1:1000
    set_up_classifier(hashtag, Xtr, Xte, Y, resultsFolder);
end

% Joins the results from the results from each hashtag
root = '/dist_vision_scratch/InstagramFood/results_raw_classifier/';
%joinResults(root);
joinResults2(root);

load('/dist_vision_scratch/InstagramFood/meta_cols/urlsAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/filenamesAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/goodAll.mat');
goodIdx = find(goodAll);
testIdx = goodIdx(2:2:end);

% Include the hashtag numbers whose results are going to be investigated
hashtags = [43, 224, 325, 306, 34, 129, 277, 97, 32, 451,...
    894, 593, 972, 934, 909, 831, 847, 413, 835, 897];
resultsRoot = 'results_raw_classifier';
% Get the best scoring images from SVM
for e = 1:numel(hashtags)
    get_svm_best_scoring(hashtags(e), urlsAll, filenamesAll, testIdx, resultsRoot)
end

% Visualize the best and worst scoring images from SVM. %This one breaks. I
% think needs an (e)?
%root = '';
root = '/dist_vision_scratch/InstagramFood/results_raw_classifier/';
for e = 1:numel(hashtags)
    visualize_svm_best_scoring(hashtags(e), root);
end

%% C2: Train a classifier: for a tag - filtering out distractor classes

% Balance training and testing sets and run classifiers
load('/dist_vision_scratch/InstagramFood/x_and_y/Xtr.mat');
load('/dist_vision_scratch/InstagramFood/x_and_y/Xte.mat');
load('/dist_vision_scratch/InstagramFood/x_and_y/Y.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/goodAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/notDistractClass.mat');
resultsFolder = 'results_no_distractors';
for hashtag = 1:1000
    set_up_classifier_remove_distr(hashtag, goodAll, notDistractClass, Xtr, Xte, Y, resultsFolder); %
end

% Joins the results from the classifiers for each hashtag 
root = '/dist_vision_scratch/InstagramFood/results_no_distractors/';
joinResults(root);

load('/dist_vision_scratch/InstagramFood/meta_cols/urlsAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/filenamesAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/goodAll.mat');
goodIdx = find(goodAll);
testIdx = goodIdx(2:2:end);

% Include the hashtag numbers whose results are going to be investigated
hashtags = [43, 224, 325, 306, 34, 129, 277, 97, 32, 451,...
    894, 593, 972, 934, 909, 831, 847, 413, 835, 897];
resultsRoot = 'results_no_distractors';

% Get the best scoring images from SVM
for e = 1:numel(hashtags)
    get_svm_best_scoring(hashtags(e), urlsAll, filenamesAll, testIdx, resultsRoot)
end

% Visualize the best scoring images from SVM
root = '';
for e = 1:numel(hashtags)
    visualize_svm_best_scoring(hashtags(e), root);
end

%% Train a classifier: for a tag - only with vgg food class images
% (this includes plate and bowl class)

load('/dist_vision_scratch/InstagramFood/x_and_y/Xtr.mat');
load('/dist_vision_scratch/InstagramFood/x_and_y/Xte.mat');
load('/dist_vision_scratch/InstagramFood/x_and_y/Y.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/goodAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/food_class.mat');
resultsFolder = 'results_food_classifier';
for hashtag = 1:1000
    set_up_classifier_food_classes(hashtag, Xtr, Xte, Y, goodAll, food_class, resultsFolder)
end

% Joins the results from the results from each hashtag 
root = '/dist_vision_scratch/InstagramFood/results_food_classifier/';
joinResults(root);

% Get the best scoring images from SVM
load('/dist_vision_scratch/InstagramFood/meta_cols/urlsAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/filenamesAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/goodAll.mat');
goodIdx = find(goodAll);
testIdx = goodIdx(2:2:end);

hashtags = [43, 224, 325, 306, 34, 129, 277, 97, 32, 451,...
    894, 593, 972, 934, 909, 831, 847, 413, 835, 897];
resultsRoot = 'results_food_classifier';
% Get the best scoring images from SVM
for e = 1:numel(hashtags)
    get_svm_best_scoring(hashtags(e), urlsAll, filenamesAll, testIdx, resultsRoot)
end

% Visualize the best scoring images from SVM
root = '';
for e = 1:numel(hashtags)
    visualize_svm_best_scoring(hashtags(e), root);
end

%% Train a classifier: for a tag - only with vgg plate and bowl classes

load('/dist_vision_scratch/InstagramFood/x_and_y/Xtr.mat');
load('/dist_vision_scratch/InstagramFood/x_and_y/Xte.mat');
load('/dist_vision_scratch/InstagramFood/x_and_y/Y.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/goodAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/plate_bowl.mat');
resultsFolder = 'results_plate_bowl_classifier';
for hashtag = 1:1000
    set_up_classifier_plate_bowl(hashtag, Xtr, Xte, Y, goodAll, plate, resultsFolder)
end
% Joins the results from the results from each hashtag
root = '/dist_vision_scratch/InstagramFood/results_plate_bowl_classifier/';
joinResults(root);

% Get the best scoring images from SVM
load('/dist_vision_scratch/InstagramFood/meta_cols/urlsAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/filenamesAll.mat');
load('/dist_vision_scratch/InstagramFood/meta_cols/goodAll.mat');
goodIdx = find(goodAll);
testIdx = goodIdx(2:2:end);

hashtags = [43, 224, 325, 306, 34, 129, 277, 97, 32, 451,...
    894, 593, 972, 934, 909, 831, 847, 413, 835, 897];
resultsRoot = 'results_plate_bowl_classifier';
% Get the best scoring images from SVM
for e = 1:numel(hashtags)
    get_svm_best_scoring(hashtags(e), urlsAll, filenamesAll, testIdx, resultsRoot)
end

% Visualize the best scoring images from SVM
root = '';
for e = 1:numel(hashtags)
    visualize_svm_best_scoring(hashtags, root);
end

%% Spatial Analysis

% Creates meta with only the instances that are reverse geocoded - includes
% only the country, continent, vgg class, tagBits
get_spatial_metas();

% Saves the most frequent tags by continent (tagNames) and country
% (tagNames_countries) in /scratch/InstagramFood/
spatial_tag_analysis();

% Get the most frequent vgg classes by continent (classes_continents) and
% by country (classes_countries)
local_vgg_analysis();

spatial_tag_correlation();

local_tag_correlation();

