%% Written by Dr. Tim Hospedales 1/2015
%%
function meta = parse_hashtags(root, fileName)

% Open and process the Metadata file into a structure array.


fid=fopen([root,'/',fileName]);

clear meta; 
for e = 1:50000     %For each post.
    url         = fgetl(fid); %L1
    if(~ischar(url)), fprintf(1,'Read %d records. Done\n', e-1); 
        break; 
    else
        meta(e).url = url;
    end
    if(~strcmp(meta(e).url(1:4),'http')), 
        fprintf('Error at %d\n',e); 
    end %If no http here, we somehow got out of sync.
    meta(e).user        = fgetl(fid); %L2
    meta(e).gps         = fgetl(fid); %L3
    next                = fgetl(fid);
    if(strcmp(next(1:7), 'NO TAGS'))
        meta(e).timeStamp   = '';
        meta(e).rawTagStr   = ''; %L5
    elseif ((~strcmp(next(1:4), '[Tag')))
        meta(e).timeStamp   = next;
        meta(e).rawTagStr   = fgetl(fid); %L5
    else
        meta(e).timeStamp   = '';
        meta(e).rawTagStr   = next; %L5
    end
    s=regexp(meta(e).rawTagStr(2:end-1), ', Tag: ', 'split'); %Make the tags into cell array.
    s{1} = s{1}(6:end);             
    meta(e).tags        = s;
    meta(e).numTags     = numel(meta(e).tags);   
    
    caption             = fgetl(fid); %L6
    next                = fgetl(fid);
    %while(isempty(next) || (length(next) > 1 && ~strcmp(next(1:2),'[C')))
    %if (length(next) > 1 && ~strcmp(next(1:2),'[C'))
    %if (next(1)
    %if (length(next) > 1 && ~strcmp(next(1:2),'[C'))
    %while(isempty(next) || next(1)~='[')
    %    caption = [caption, next];
    %    next = fgetl(fid);
    %end
    while(true)
        if((length(next)==2) && strcmp(next(1:2),'[]')), break; end
        if((length(next)>=8) && strcmp(next(1:8),'[Comment')), break; end
        %Terminate if next==[] or next==[Comment
        caption = [caption, next];
        next = fgetl(fid);
    end
    meta(e).caption     = caption;
    followComments      = next; %L7
    
    while(followComments(end)~=']')
        next = fgetl(fid);
        followComments  = [followComments, next];
    end

    meta(e).followComments = followComments; 
    numComments         = fgetl(fid); %L8
    meta(e).numComments = str2double(numComments(16:end)); % Convert string to actual number.
    meta(e).likingUsers = fgetl(fid); %L9
    numLikes            = fgetl(fid); %L10
    meta(e).numLikes    = str2double(numLikes(13:end));  %Convert string to actual number
end

fclose(fid);

%{
% Get the number of unique tags used.
[tagDict,ia,ic]=unique([meta.tags],'stable'); %tagDict is the dictionary of tags.
fprintf(1, 'Number of unique tags: %d\n', numel(tagDict));

% Convert the tag datastructure format
% For each post, convert cellarray of text tags to a list of tag IDs, and a binary indicator matrix.
ctr = 1;
for e = 1 : num_entries
    meta(e).tagIdx = ismember(tagDict, meta(e).tags);   %Convert text list per image to a list of numeric IDs.
    meta(e).tagBits = false(1,numel(tagDict));
    meta(e).tagBits(meta(e).tagIdx) = true;             %List of numeric IDs to a binary vector per image.
    sort(meta(e).tags);                                 
    sort(tagDict(meta(e).tagIdx));                      %Verify no errors.
    tagBits(e, 1:numel(tagDict)) = meta(e).tagBits;      % Create a binary indicator matrix for each tag from the meta.tagBits
end
%}

%{
% Read the latitude and longitude coordinates using the google maps API to
% determine the country and continent
for e = 1:num_entries
    if (~strncmp(meta(e).gps,'NO LOCATION', 11))
        gpsSplit = strsplit(meta(e).gps, {'(', ',', ')', ' '});
        if (size(gpsSplit)>=5)
            str = {'AD','Europe';'AE','Asia';'AF','Asia';'AG','North America';'AI','North America';'AL','Europe';'AM','Asia';'AN','North America';'AO','Africa';'AQ','Antarctica';'AR','South America';'AS','Australia';'AT','Europe';'AU','Australia';'AW','North America';'AZ','Asia';'BA','Europe';'BB','North America';'BD','Asia';'BE','Europe';'BF','Africa';'BG','Europe';'BH','Asia';'BI','Africa';'BJ','Africa';'BM','North America';'BN','Asia';'BO','South America';'BR','South America';'BS','North America';'BT','Asia';'BW','Africa';'BY','Europe';'BZ','North America';'CA','North America';'CC','Asia';'CD','Africa';'CF','Africa';'CG','Africa';'CH','Europe';'CI','Africa';'CK','Australia';'CL','South America';'CM','Africa';'CN','Asia';'CO','South America';'CR','North America';'CU','North America';'CV','Africa';'CX','Asia';'CY','Asia';'CZ','Europe';'DE','Europe';'DJ','Africa';'DK','Europe';'DM','North America';'DO','North America';'DZ','Africa';'EC','South America';'EE','Europe';'EG','Africa';'EH','Africa';'ER','Africa';'ES','Europe';'ET','Africa';'FI','Europe';'FJ','Australia';'FK','South America';'FM','Australia';'FO','Europe';'FR','Europe';'GA','Africa';'GB','Europe';'GD','North America';'GE','Asia';'GF','South America';'GG','Europe';'GH','Africa';'GI','Europe';'GL','North America';'GM','Africa';'GN','Africa';'GP','North America';'GQ','Africa';'GR','Europe';'GS','Antarctica';'GT','North America';'GU','Australia';'GW','Africa';'GY','South America';'HK','Asia';'HN','North America';'HR','Europe';'HT','North America';'HU','Europe';'ID','Asia';'IE','Europe';'IL','Asia';'IM','Europe';'IN','Asia';'IO','Asia';'IQ','Asia';'IR','Asia';'IS','Europe';'IT','Europe';'JE','Europe';'JM','North America';'JO','Asia';'JP','Asia';'KE','Africa';'KG','Asia';'KH','Asia';'KI','Australia';'KM','Africa';'KN','North America';'KP','Asia';'KR','Asia';'KW','Asia';'KY','North America';'KZ','Asia';'LA','Asia';'LB','Asia';'LC','North America';'LI','Europe';'LK','Asia';'LR','Africa';'LS','Africa';'LT','Europe';'LU','Europe';'LV','Europe';'LY','Africa';'MA','Africa';'MC','Europe';'MD','Europe';'ME','Europe';'MG','Africa';'MH','Australia';'MK','Europe';'ML','Africa';'MM','Asia';'MN','Asia';'MO','Asia';'MP','Australia';'MQ','North America';'MR','Africa';'MS','North America';'MT','Europe';'MU','Africa';'MV','Asia';'MW','Africa';'MX','North America';'MY','Asia';'MZ','Africa';'NA','Africa';'NC','Australia';'NE','Africa';'NF','Australia';'NG','Africa';'NI','North America';'NL','Europe';'NO','Europe';'NP','Asia';'NR','Australia';'NU','Australia';'NZ','Australia';'OM','Asia';'PA','North America';'PE','South America';'PF','Australia';'PG','Australia';'PH','Asia';'PK','Asia';'PL','Europe';'PM','North America';'PN','Australia';'PR','North America';'PS','Asia';'PT','Europe';'PW','Australia';'PY','South America';'QA','Asia';'RE','Africa';'RO','Europe';'RS','Europe';'RU','Europe';'RW','Africa';'SA','Asia';'SB','Australia';'SC','Africa';'SD','Africa';'SE','Europe';'SG','Asia';'SH','Africa';'SI','Europe';'SJ','Europe';'SK','Europe';'SL','Africa';'SM','Europe';'SN','Africa';'SO','Africa';'SR','South America';'ST','Africa';'SV','North America';'SY','Asia';'SZ','Africa';'TC','North America';'TD','Africa';'TF','Antarctica';'TG','Africa';'TH','Asia';'TJ','Asia';'TK','Australia';'TM','Asia';'TN','Africa';'TO','Australia';'TR','Asia';'TT','North America';'TV','Australia';'TW','Asia';'TZ','Africa';'UA','Europe';'UG','Africa';'US','North America';'UY','South America';'UZ','Asia';'VC','North America';'VE','South America';'VG','North America';'VI','North America';'VN','Asia';'VU','Australia';'WF','Australia';'WS','Australia';'YE','Asia';'YT','Africa';'ZA','Africa';'ZM','Africa';'ZW','Africa'};
            url = horzcat('http://maps.googleapis.com/maps/api/geocode/xml?latlng=', char(gpsSplit(4)), ',', char(gpsSplit(5)), '&sensor=true');
            queryResult = urlread(url);
            %exp = '<s\<[A-Z]{2}\></short_name>\s<type>country</type>'
            exp = '\<[A-Z]{2}\></short_name>';
            clear startIndex;
            startIndex = regexp(queryResult, exp);
            countryName = queryResult(startIndex(2):startIndex(2)+1);
            meta(e).country = countryName;
            strfind(cellstr(str), countryName);
            continent = char(str(find(strcmp(countryName, str)), 2));
            meta(e).continent = continent;
        end
    end
end


% Get most frequent tags.
allTagLabs = cat(1,meta.tagBits);
labFreq = sum(allTagLabs,1);                            %Get overall tag frequency by summing the per-image binary vector.
[sval,sidx] = sort(labFreq,'descend');
disp('Top 25 frequent tags:')
disp('rank: frequency: tag');
for s = 1 : 25
    fprintf(1,'%02d : %d : %s\n', s, sval(s), tagDict{sidx(s)});
end

% Only get the top 200 most popular tags in the binary indicator matrix
topTagIdx = sort(sidx(1:200));
for e = 1 : numel(topTagIdx)
    topTagBits(:, e) = tagBits(:, topTagIdx(e));   
end

% Get the covariance of the top 25 tags binary indicator matrix and plot their correlation
C = cov(topTagBits);
imagesc(C); colorbar;
C2 = C.*(1-eye(200)); imagesc(C2); colorbar;

%[sc,sci]=sort(C2(:),'descend');
%sc(1)
%sci(1)
%[i,j]=ind2sub([numel(topTagIdx),numel(topTagIdx)],)
%}


