function run_classifier(hashtag, Xtr, Xte, Ytr, Yte, resultsFolder)
% Run classifier without cross validation

%{
LIBLINEAR: A Library for Large Linear Classification J. Mach. Learn. Res.,
Vol. 9 (June 2008), pp. 1871-1874 by Rong E. Fan, Kai W. Chang,
Cho J. Hsieh, Xiang R. Wang, Chih J. Lin
%}

% Column 1 = hashtag number (by tagDict), Column 2 = test accuracy
% Column 3 = Normalized Confusion Matrix Accuracy
% Column 4 = libsvm score
tic;
classifier_results = zeros(1,3);
classifier_results(1,1) = hashtag;

Xtr = sparse(double(Xtr));
Xte = sparse(double(Xte));

%addpath('liblinear-2.1/matlab/'); cmd = '-s 2'; % Singlecore.
addpath('liblinear-multicore-2.1-2/matlab/'); cmd = '-s 2 -n 16'; % Multicore

fprintf(1, 'Classify test dataset - %d\n', hashtag);
bestcv=train(Ytr,Xtr,cmd);      %Train linear SVM (liblinear)
[pred,acc,score]=predict(Yte,Xte,bestcv);          %Test SVM (liblinear)

fprintf(1,'Test accuracy: %2.2f\n', acc(1));
classifier_results(1, 2) = acc(1);
conf = confusionmat(Yte,pred);      %Confusion matrix
normConfAcc = 100*((conf(1,1)/sum(conf(1,:))+ conf(2,2)/sum(conf(2,:)))/2);
fprintf(1, 'Normalized Confusion Matrix Accuracy: %2.2f\n', normConfAcc);
classifier_results(1, 3) = normConfAcc;

outFileRes = ['/dist_vision_scratch/InstagramFood/', resultsFolder, '/classifier_results/classifier_results_',num2str(hashtag),'.mat'];
save(outFileRes, 'classifier_results');

outFile = ['/dist_vision_scratch/InstagramFood/', resultsFolder, '/predict/pred',num2str(hashtag),'.mat'];
save(outFile,'pred');

outFile = ['/dist_vision_scratch/InstagramFood/', resultsFolder, '/scores/score',num2str(hashtag),'.mat'];
save(outFile,'score');
t1 = toc;
fprintf(1, 'Time to process one hashtag = %d\n', t1);
