function spatial_tag_analysis()

load('/dist_vision_scratch/InstagramFood/spatial_analysis/joined_spatial_metas.mat');

% Saves the tagBits of the joined_spatial_metas into a matrix
for e = 1:length(joined_spatial_metas)
    tagBitsMat(e,:) = vec2mat(joined_spatial_metas(e).tagBits(1:500), 500);
    fprintf(1, 'Entry: %d\n', e);
    if(mod(e, 10000) == 0)
        save('/dist_vision_scratch/InstagramFood/spatial_analysis/tagBitsMat.mat', 'tagBitsMat');
    end
end

save('/dist_vision_scratch/InstagramFood/spatial_analysis/tagBitsMat.mat', 'tagBitsMat');

%% Get the most frequently occurring VGG classes for each continent

fields = {'as', 'af', 'au', 'ce', 'eu', 'na', 'sa'};
continents = {'Asia', 'Africa','Australia','Central America and the Caribbean', 'Europe','North America','South America'};
bestCatIDs = {joined_spatial_metas.bestCatID};

% idx = structure array by continent of indices of joined_spatial_metas that belong to
% each continent
% meta = structure array by continent of joined_spatial_metas instances
% cats = structure array by continent of the vgg classes belonging to each
% continent
for e = 1:numel(fields)
    idx.(char(fields(e))) = find(cellfun(@(x)isequal(x,(char(continents(e)))),{joined_spatial_metas.continent}));
    meta.(char(fields(e))) = joined_spatial_metas(idx.(char(fields(e))));
    cats.(char(fields(e))) = cell2mat(bestCatIDs(idx.(char(fields(e)))))';
end

save('/dist_vision_scratch/InstagramFood/spatial_analysis/idx.mat', 'idx');
save('/dist_vision_scratch/InstagramFood/spatial_analysis/meta.mat', 'meta');
save('/dist_vision_scratch/InstagramFood/spatial_analysis/cats.mat', 'cats');

%% Get the most frequently occurring tags for each continent

for f = 1:numel(fields)
    bits.(char(fields(f))) = tagBitsMat(idx.(char(fields(f))),:);
end
save('/dist_vision_scratch/InstagramFood/spatial_analysis/bits.mat', 'bits');

for f = 1:numel(fields)
    for g = 1:500
        sumBits.(char(fields(f)))(g) = sum(bits.(char(fields(f)))(:,g));  
    end
    [sortedTagFreq.(char(fields(f))), tagsIdx.(char(fields(f)))] = sort(sumBits.(char(fields(f))),'descend');
end

load('/dist_vision_scratch/InstagramFood/tagDicts/tagDict.mat');

% tagNames is a structure array by continent by tagName 
tagNames.as = tagDict(tagsIdx.as);
tagNames.af = tagDict(tagsIdx.af);
tagNames.au = tagDict(tagsIdx.au);
tagNames.eu = tagDict(tagsIdx.eu);
tagNames.na = tagDict(tagsIdx.na);
tagNames.sa = tagDict(tagsIdx.sa);

save('/dist_vision_scratch/InstagramFood/spatial_analysis/sortedTagFreq.mat', 'sortedTagFreq');
save('/dist_vision_scratch/InstagramFood/spatial_analysis/tagsIdx.mat', 'tagsIdx');
save('/dist_vision_scratch/InstagramFood/spatial_analysis/tagNames.mat', 'tagNames');

%% Get the most frequently occurring VGG classes for each of the top 20 countries

fields_countries = {'US', 'AU', 'IT', 'GB', 'ID', 'PH', 'SG', 'TH', 'CA', 'TR'...
          'JP', 'DE', 'BR', 'MY', 'HK', 'ES', 'RU', 'FR', 'KR', 'TW'};
      
for e = 1:numel(fields_countries)
    idx_countries.(char(fields_countries(e))) = find(cellfun(@(x)isequal(x,(char(fields_countries(e)))),{joined_spatial_metas.country}));
    cats_countries.(char(fields_countries(e))) = cell2mat(bestCatIDs(idx_countries.(char(fields_countries(e)))))';
end

save('/dist_vision_scratch/InstagramFood/spatial_analysis/idx_countries.mat', 'idx_countries');
save('/dist_vision_scratch/InstagramFood/spatial_analysis/cats_countries.mat', 'cats_countries');

%% Get the most frequently occurring tags for each of the top 20 countries

for f = 1:numel(fields_countries)
    bits_countries.(char(fields_countries(f))) = tagBitsMat(idx_countries.(char(fields_countries(f))),:);
end
save('/dist_vision_scratch/InstagramFood/spatial_analysis/bits_countries.mat', 'bits_countries');

for f = 1:numel(fields_countries)
    for g = 1:500
        sumBits_countries.(char(fields_countries(f)))(g) = sum(bits_countries.(char(fields_countries(f)))(:,g));  
    end
    [sortedTagFreq_countries.(char(fields_countries(f))), tagsIdx_countries.(char(fields_countries(f)))] = sort(sumBits_countries.(char(fields_countries(f))),'descend');
end

load('/dist_vision_scratch/InstagramFood/tagDicts/tagDict.mat');

% tagNames is a structure array by continent by tagName 
for e = 1:numel(fields_countries)
    tagNames_countries.(char(fields_countries(e))) = tagDict(tagsIdx_countries.(char(fields_countries(e))));
end

save('/dist_vision_scratch/InstagramFood/spatial_analysis/sortedTagFreq_countries.mat', 'sortedTagFreq_countries');
save('/dist_vision_scratch/InstagramFood/spatial_analysis/tagsIdx_countries.mat', 'tagsIdx_countries');
save('/dist_vision_scratch/InstagramFood/spatial_analysis/tagNames_countries.mat', 'tagNames_countries');
